/*******************************************************************************
*
*		Filename:	game.c.
*
*		Project:	Dual Video Reel Demo.
*
*		Author:		Jon Carroll.
*
*		Created:	19 July 2005.
*
*
*******************************************************************************/

/*******************************************************************************
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
*******************************************************************************/

/*******************************************************************************
*								Include files
*******************************************************************************/

#include	"hardware.h"
#include	"peripherals.h"
#include	"interface.h"
#include	"fpga.h"
#include	"config.h"
#include	"game.h"

#include	"game.sp"



/*******************************************************************************
*								Definitions
*******************************************************************************/

											/* the number of bytes per a pixel
											   for 16 bit colour this is 2 */
#define BYTES_PER_PIXEL 	2
											/* the pixel moved each step of the
											   reel */
#define	PIXELS_PER_STEP1	32

											/* the position of the reel 1 */
#define REEL1XS1			176
#define REEL2XS1			(REEL1XS1 + 128 + 8)
#define REEL3XS1			(REEL2XS1 + 128 + 8)
#define REEL4XS1			(REEL3XS1 + 128 + 8)
#define REEL5XS1			(REEL4XS1 + 128 + 8)
#define REELYS1				124

#define TRANS_COLOUR		0x0200
#define DISABLE				0
#define	SCREEN1_WIDTH		1024
#define	SCREEN2_WIDTH		1024
#define	SCREEN1_HEIGHT		768
#define	SCREEN2_HEIGHT		768

											/* the reel window height & width */
#define	REEL_WIN_HEIGHTS1	384
#define	REEL_WIN_WIDTHS1	128
											/* reel band length */
#define REEL_BAND_HEIGHTS1	1536
											/* total number of reels
											   on screen 1 */
#define TOTAL_REELS_SCREEN1	5

											/* the pixel moved each step of the
											   reel */
#define	PIXELS_PER_STEP2	64


#define REEL1XS2			120
#define REEL2XS2			(REEL1XS2 + 256 + 8)
#define REEL3XS2			(REEL2XS2 + 256 + 8)
#define REELYS2				184
											/* the reel window
											   height & width */
#define	REEL_WIN_HEIGHTS2	512
#define	REEL_WIN_WIDTHS2	256
											/* reel band length */
#define REEL_BAND_HEIGHTS2	3072
											/* total number of reels
											   on screen 2 */
#define	TOTAL_REELS_SCREEN2	3


											/* the reel spin states */
	enum
	{
		WAITING_STATE,
		STEP_BACK_STATE,
		STEP_FORE_STATE,
		SPIN_STATE,

		TOTAL_SPIN_STATES
	};
											/* the reel bounce states */
	enum
	{
		NORMAL_STATE,
		DOWN_STATE,
		UP_STATE,
		FINISH_STATE,

		TOTAL_BOUNCE_STATES
	};

/*******************************************************************************
*							variable declarations
*******************************************************************************/

											/* memory address to the cremson memory */
	static ULONG screen1mem[S1_TOTAL_GRAPHICS];

											/* memory address to the cremson memory */
	static ULONG screen2mem[S2_TOTAL_GRAPHICS];
											/* the current position of the reels */
	static ULONG S1ReelPosition[TOTAL_REELS_SCREEN1];
											/* the current number of steps to go */
	static ULONG S1StepsToGo[TOTAL_REELS_SCREEN1];
											/* screen 1 state */
	static UBYTE S1State;
											/* screen 1 state timer */
	static TIMER S1StateTimer;
											/* screen 1 reel bounce */
	static UBYTE S1ReelBounceState[TOTAL_REELS_SCREEN1];
											/* the current position of the reels */
	static ULONG S2ReelPosition[TOTAL_REELS_SCREEN2];
											/* the current number of steps to go */
	static ULONG S2StepsToGo[TOTAL_REELS_SCREEN2];
											/* screen 1 state */
	static UBYTE S2State;
											/* screen 1 state timer */
	static TIMER S2StateTimer;
											/* screen 1 reel bounce */
	static UBYTE S2ReelBounceState[TOTAL_REELS_SCREEN2];




/*******************************************************************************
*						Non-volatile variable declarations
*******************************************************************************/

/*******************************************************************************
*							constant declarations
*******************************************************************************/

											/* The machine identity, this needs
											   to be different for each new machine
											*/
	const LONG MachineId = 0x00000002;



	static const BYTE * S1Filenames[S1_LAST_GRAPHIC] =
	{
		"",
		"",
		"",
		"background2.bmp",
		"reel1.bmp"
	};

	static const BYTE * S2Filenames[S2_LAST_GRAPHIC] =
	{
		"",
		"",
		"",
		"background.bmp",
		"reel6.bmp"
	};



											/* reel position */
	static const ULONG ReelWindowS1[TOTAL_REELS_SCREEN1] =
	{
		REEL1XS1,
		REEL2XS1,
		REEL3XS1,
		REEL4XS1,
		REEL5XS1
	};

	static const ULONG ReelWindowS2[TOTAL_REELS_SCREEN2] =
	{
		REEL1XS2,
		REEL2XS2,
		REEL3XS2
	};



/*******************************************************************************
*								Functions
*******************************************************************************/


/*******************************************************************************
Name......... _putchar()
Author....... Marc Adams
Date......... 15/09/03
Description.. Fuction stub to satisfy GNU compiler
Parameters... None
Return....... None
History...... 15/09/03 - Code tidy up  - MSA
*******************************************************************************/
void	_putchar()
{

}

/*-------------------------------------------------------------------
Name......... Game()
Author....... Jon Carroll
Date......... 20/07/05
Description.. Libraries initialise system then branch to here
			  The main entry point in to the game
Parameters... None
Return....... None
History...... None
-------------------------------------------------------------------*/
void Game()
{

												/* clear the reel position */
	clearreelpositions();
												/* initlise the Video system */
	initvideohardware();
												/* load graphics */
	loadgraphics();
												/* start demo */
	startdemo();
}


/*******************************************************************************
Name......... initvideohardware
Author....... Jon Carroll
Date......... 19/07/05
Description.. initlises the screen hardware and loads basic fonts
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void initvideohardware(void)
{
											/* set up the shorten versions */
	screen1 = &Calypso32Dual0Device;
	screen2 = &Calypso32Dual1Device;

											/* turn the video off */
	CalypsoVideoOff(screen1);
	CalypsoVideoOff(screen2);

											/* Disable all layer */

	CalypsoDisableLayer(screen1, ALL_SELECT);
	CalypsoDisableLayer(screen2, ALL_SELECT);
											/* init the calypso drivers
											   for both screens

											   screen 1 = Bottom reels
											   screen 2 = prize table
											*/
	CalypsoInitDisplayMode(screen1, Calypso1024x768);
	CalypsoInitDisplayMode(screen2, Calypso1024x768);

											/* Disable all layer */

	CalypsoDisableLayer(screen1, ALL_SELECT);
	CalypsoDisableLayer(screen2, ALL_SELECT);

											/* clear the animation list */
	CalypsoClearAnimationList(screen1);
	CalypsoClearAnimationList(screen2);
											/* clear alpha blending */
	CalypsoAlphaBlend(screen1, DISABLE);
	CalypsoAlphaBlend(screen2, DISABLE);

											/* process screen 1 first */

											/* set the base layer
											start addresses */
	screen1mem[S1_BASE_MEMORY] = 0;
											/* initlise the
											background layer */
											/* returns the middle
											layer start address */
	screen1mem[S1_MIDDLE_MEMORY] =
			CalypsoInitLayer(screen1, BL_LAYER, DIRECT_COLOR,
			DOUBLEBUFFER, TRANS_COLOUR, SCREEN1_WIDTH,
			SCREEN1_HEIGHT, screen1mem[S1_BASE_MEMORY]);

	screen1mem[S1_SCRATCH_MEMORY] =
			CalypsoInitLayer(screen1, ML_LAYER, DIRECT_COLOR,
			DOUBLEBUFFER, TRANS_COLOUR, SCREEN1_WIDTH,
			SCREEN1_HEIGHT, screen1mem[S1_MIDDLE_MEMORY]);

											/* create scratch memory
											to load BMP files */
	screen1mem[S1_FIRST_GRAPHIC] =
				(SCREEN1_WIDTH * SCREEN1_HEIGHT * 4) +
				screen1mem[S1_SCRATCH_MEMORY];



											/* clear the layers */
											/* of both frames */
	CalypsoClearLayer(screen1, BL_LAYER, DOUBLEBUFFER_FRAME0);
	CalypsoClearLayer(screen1, BL_LAYER, DOUBLEBUFFER_FRAME1);
	CalypsoClearLayer(screen1, ML_LAYER, DOUBLEBUFFER_FRAME0);
	CalypsoClearLayer(screen1, ML_LAYER, DOUBLEBUFFER_FRAME1);


											/* process screen 2 */

											/* set the base layer
											start addresses */
	screen2mem[S2_BASE_MEMORY] = 0;
											/* initlise the
											background layer */
											/* returns the middle
											layer start address */
	screen2mem[S2_MIDDLE_MEMORY] =
			CalypsoInitLayer(screen2, BL_LAYER, DIRECT_COLOR,
			DOUBLEBUFFER, TRANS_COLOUR, SCREEN2_WIDTH,
			SCREEN2_HEIGHT, screen2mem[S2_BASE_MEMORY]);

	screen2mem[S2_SCRATCH_MEMORY] =
			CalypsoInitLayer(screen2, ML_LAYER, DIRECT_COLOR,
			DOUBLEBUFFER, TRANS_COLOUR, SCREEN2_WIDTH,
			SCREEN2_HEIGHT, screen2mem[S2_MIDDLE_MEMORY]);

											/* create scratch memory
											to load BMP files */
	screen2mem[S2_FIRST_GRAPHIC] =
				(SCREEN2_WIDTH * SCREEN2_HEIGHT * 4) +
				screen2mem[S2_SCRATCH_MEMORY];


											/* clear the layers */
											/* of both frames */

	CalypsoClearLayer(screen2, BL_LAYER, DOUBLEBUFFER_FRAME0);
	CalypsoClearLayer(screen2, BL_LAYER, DOUBLEBUFFER_FRAME1);
	CalypsoClearLayer(screen2, ML_LAYER, DOUBLEBUFFER_FRAME0);
	CalypsoClearLayer(screen2, ML_LAYER, DOUBLEBUFFER_FRAME1);


											/* enable the console and
											   the base layers */
	CalypsoEnableLayer(screen1, BASE_SELECT | MIDDLE_SELECT);
	CalypsoEnableLayer(screen2, BASE_SELECT | MIDDLE_SELECT);

											/* set the frame rate to manual*/
	CalypsoSetFrameRate(screen1, FRAME_0HZ);
	CalypsoSetFrameRate(screen2, FRAME_0HZ);

											/* enable double buffering */
	CalypsoEnableDoubleBuffer(screen1);
	CalypsoEnableDoubleBuffer(screen2);
											/* turn video on */
	CalypsoVideoOn(screen1);
	CalypsoVideoOn(screen2);
											/* advance two frames */
											/* to flush all the
											   display lists */

	AdvanceFrames();
	AdvanceFrames();
}


/*******************************************************************************
Name......... loadgraphics
Author....... Jon Carroll
Date......... 20/07/05
Description.. mounts and loads the graphics of the compact flash
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void loadgraphics(void)
{
	UWORD i;
	BMPFILEINFO bmpinfo;
	ULONG bmptmpptr;
	ULONG videoaddress;
	UBYTE sucess;
	ULONG videobaseaddress;
											/* mount the compact flash
											   card */
	if (MountFAT32(&ATADevice))
	{
											/* Failed to mount the CF*/
	}


											/* change the directory */
	ChangeDir(&ATADevice, "\\");
	ChangeDir(&ATADevice, "graphics");

											/* set the video base address */
	videobaseaddress = CALYPSO32DUAL0_BASE;
											/* set the scratch memory
											   pointer */
	bmptmpptr = screen1mem[S1_SCRATCH_MEMORY] + videobaseaddress;

											/* initilse the compact
											   flash device */
	for (i = S1_FIRST_GRAPHIC; i < S1_LAST_GRAPHIC; i++)
	{
											/* set the video memory storage
											   address */
		videoaddress = screen1mem[i];
											/* load graphics into video
											   memory */
											/* Knowledge Base */
		sucess = LoadMsWinBmp24(&ATADevice, S1Filenames[i], &bmpinfo,
						 (UWORD *)bmptmpptr, (videoaddress + videobaseaddress));

											/* compute the next free
											   address space */
		screen1mem[i + 1] = videoaddress +
					(bmpinfo.imagewidth * bmpinfo.imageheight * BYTES_PER_PIXEL);
	}

											/* load the graphics for screen 2*/

											/* set the video base address */
	videobaseaddress = CALYPSO32DUAL1_BASE;
											/* set the scratch memory
											   pointer */
	bmptmpptr = screen2mem[S2_SCRATCH_MEMORY] + videobaseaddress;

											/* initilse the compact
											   flash device */
	for (i = S2_FIRST_GRAPHIC; i < S2_LAST_GRAPHIC; i++)
	{
											/* set the video memory storage
											   address */
		videoaddress = screen2mem[i];
											/* load graphics into video
											   memory */
											/* Knowledge Base */
		sucess = LoadMsWinBmp24(&ATADevice, S2Filenames[i], &bmpinfo,
						 (UWORD *)bmptmpptr, (videoaddress + videobaseaddress));

											/* compute the next free
											   address space */
		screen2mem[i + 1] = videoaddress +
					(bmpinfo.imagewidth * bmpinfo.imageheight
						* BYTES_PER_PIXEL);
	}
}

/*******************************************************************************
Author........ Alan Austin
Last Modified. 12 May 2004
Description... Wait for n video frames
..............
Parameters.... no of frames
..............
Return Value..
..............
*******************************************************************************/

BYTE WaitNFrames(const DEVICE* calypso32device, UBYTE nframes)
{
	UBYTE i;
	WORD frame_number;

	for(i = 0; i < nframes; i++)
	{
		frame_number = CalypsoGetFrameNumber(calypso32device);

		while (frame_number == CalypsoGetFrameNumber(calypso32device));
	}

	return(CalypsoGetDoubleBufferFrame(calypso32device));
}


/*******************************************************************************
Name......... startdemo()
Author....... Jon carroll
Date......... 25/07/05
Description.. start of reel demo
Parameters... None
Return....... None
History...... None
*******************************************************************************/
void startdemo(void)
{
												/* draw the backgroud
												   on screen 1 */
	drawbackground1();
												/* draw the background
												   on screen 2 */
	drawbackground2();

	while(1)
	{
												/* update the display */
		UpdateDisplayScreen1();
		UpdateDisplayScreen2();
												/* display the reels */
		displayreelsscreen1();
		displayreelsscreen2();
												/* advance both screens */

		AdvanceFrames();
	}
}

/*******************************************************************************
Name......... displayreelsscreen1()
Author....... Jon carroll
Date......... 25/07/05
Description.. display the current reels position on screen 1
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void displayreelsscreen1(void)
{
	volatile ULONG specialheight;
	UBYTE Reel;

												/* for all reels */
	for (Reel = 0; Reel < TOTAL_REELS_SCREEN1; Reel++)
	{
												/* output Reel  */
		if (S1ReelPosition[Reel] > (REEL_BAND_HEIGHTS1 - REEL_WIN_HEIGHTS1))
		{
												/* compute the special height */
			specialheight = REEL_WIN_HEIGHTS1 -
				(REEL_BAND_HEIGHTS1 - S1ReelPosition[Reel]);

												/* output top of reel */
			CalypsoPutPartialSprite(screen1,
					BL_LAYER, DOUBLEBUFFER_AUTO, ReelWindowS1[Reel], REELYS1,
					screen1mem[S1_REEL1], REEL_WIN_WIDTHS1,
					REEL_WIN_HEIGHTS1 - specialheight ,
					NORMAL_MODE, S1ReelPosition[Reel], 0);

												/* determine special
												height is not 0 */
			if (specialheight)
			{
												/* output bottom of reel */
				CalypsoPutPartialSprite(screen1,
						BL_LAYER, DOUBLEBUFFER_AUTO, ReelWindowS1[Reel],
						REELYS1 + (REEL_WIN_HEIGHTS1 - specialheight),
						screen1mem[S1_REEL1], REEL_WIN_WIDTHS1, specialheight,
						NORMAL_MODE, 0, 0);
			}
		}
		else

		{
												/* output the first symbol */
			CalypsoPutPartialSprite(screen1,
					BL_LAYER, DOUBLEBUFFER_AUTO, ReelWindowS1[Reel], REELYS1,
					screen1mem[S1_REEL1], REEL_WIN_WIDTHS1, REEL_WIN_HEIGHTS1,
					NORMAL_MODE, S1ReelPosition[Reel], 0);
		}
	}
}

/*******************************************************************************
Name......... displayreelsscreen2()
Author....... Jon carroll
Date......... 25/07/05
Description.. display the current reels position on screen 2
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void displayreelsscreen2(void)
{
	ULONG specialheight;
	UBYTE Reel;

												/* for all reels */
	for (Reel = 0; Reel < TOTAL_REELS_SCREEN2; Reel++)
	{
												/* output Reel  */
		if (S2ReelPosition[Reel] > (REEL_BAND_HEIGHTS2 - REEL_WIN_HEIGHTS2))
		{
												/* compute the special height */
			specialheight = REEL_WIN_HEIGHTS2 -
					(REEL_BAND_HEIGHTS2 - S2ReelPosition[Reel]);

												/* output top of reel */
			CalypsoPutPartialSprite(screen2, BL_LAYER,
					DOUBLEBUFFER_AUTO, ReelWindowS2[Reel], REELYS2,
					screen2mem[S2_REEL6], REEL_WIN_WIDTHS2,
					REEL_WIN_HEIGHTS2 - specialheight ,
					NORMAL_MODE, S2ReelPosition[Reel], 0);

												/* determine special height is not 0 */
			if (specialheight)
			{
												/* output bottom of reel */
				CalypsoPutPartialSprite(screen2, BL_LAYER, DOUBLEBUFFER_AUTO,
						ReelWindowS2[Reel],
						REELYS2 + (REEL_WIN_HEIGHTS2 - specialheight),
						screen2mem[S2_REEL6], REEL_WIN_WIDTHS2, specialheight,
						NORMAL_MODE, 0, 0);
			}
		}
		else

		{
												/* output the first symbol */
			CalypsoPutPartialSprite(screen2, BL_LAYER, DOUBLEBUFFER_AUTO,
					ReelWindowS2[Reel], REELYS2,
					screen1mem[S2_REEL6], REEL_WIN_WIDTHS2, REEL_WIN_HEIGHTS2,
					NORMAL_MODE, S2ReelPosition[Reel], 0);
		}
	}
}

/*******************************************************************************
Name......... drawbackground1()
Author....... Jon carroll
Date......... 27/07/05
Description.. redraw the screen background on screen 1
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void drawbackground1(void)
{
												/* redraw the background */
	CalypsoPutSprite(screen1, BL_LAYER, DOUBLEBUFFER_FRAME0, 0, 0,
			screen1mem[S1_BACKGROUND], 1024, 768, NORMAL_MODE);

	CalypsoPutSprite(screen1, BL_LAYER, DOUBLEBUFFER_FRAME1, 0, 0,
			screen1mem[S1_BACKGROUND], 1024, 768, NORMAL_MODE);
}

/*******************************************************************************
Name......... drawbackground2()
Author....... Jon carroll
Date......... 27/07/05
Description.. redraw the screen background on screen 2
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void drawbackground2(void)
{
												/* redraw the background */
	CalypsoPutSprite(screen2, BL_LAYER, DOUBLEBUFFER_FRAME0, 0, 0,
			screen1mem[S2_BACKGROUND], 1024, 768, NORMAL_MODE);

	CalypsoPutSprite(screen2, BL_LAYER, DOUBLEBUFFER_FRAME1, 0, 0,
			screen1mem[S2_BACKGROUND], 1024, 768, NORMAL_MODE);
}

/*******************************************************************************
Name......... CalypsoPutSprite()
Author....... Jon carroll
Date......... 27/07/05
Description.. start of reel demo
Parameters... None
Return....... None
History...... None
*******************************************************************************/

WORD CalypsoPutPartialSprite(const DEVICE* dev, LONG layer_select, BYTE doublebuffer_frame,
		LONG x, LONG y, LONG image_start, LONG xsize, LONG ysize, LONG mode,
		LONG partialHeight, LONG particalWidth)
{
	LONG memstart;

												/* width is not implemented */
	particalWidth = 0;

												/* compute the start of the height */
	memstart = image_start + (partialHeight * xsize * BYTES_PER_PIXEL);

												/* output the sprite */
	return (CalypsoPutSprite(dev, layer_select, doublebuffer_frame,
		x, y, memstart, xsize, ysize, mode));
}


/*******************************************************************************
Name......... clearreelpositions()
Author....... Jon carroll
Date......... 27/07/05
Description.. clear the reel positions
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void clearreelpositions(void)
{
										/* clear all the reel positions */
	S1ReelPosition[0] = 0;
	S1ReelPosition[1] = 128;
	S1ReelPosition[2] = 256;
	S1ReelPosition[3] = 640;
	S1ReelPosition[4] = 384;
										/* set the screen 1 state */
	S1State = WAITING_STATE;
										/* set steps to go */
	S1StepsToGo[0] = 3;
	S1StepsToGo[1] = 3;
	S1StepsToGo[2] = 3;
	S1StepsToGo[3] = 3;
	S1StepsToGo[4] = 3;


										/* clear all the reel positions */
	S2ReelPosition[0] = 128;
	S2ReelPosition[1] = 128 + 256;
	S2ReelPosition[2] = 128 + (256 *2);
										/* set the screen 1 state */
	S2State = WAITING_STATE;
										/* set steps to go */
	S2StepsToGo[0] = 3;
	S2StepsToGo[1] = 3;
	S2StepsToGo[2] = 3;

}

/*******************************************************************************
Name......... StepReelForeWardsScreen1
Author....... Jon carroll
Date......... 27/07/05
Description.. Steps the reels up
Parameters... 1) The reel
Return....... None
History...... None
*******************************************************************************/

void StepReelForewardsScreen1(UBYTE Reel)
{
	if (S1ReelPosition[Reel] < PIXELS_PER_STEP1)
	{
		S1ReelPosition[Reel] = (REEL_BAND_HEIGHTS1 - S1ReelPosition[Reel]) - PIXELS_PER_STEP1;
	}
	else
	{
		S1ReelPosition[Reel] -= PIXELS_PER_STEP1;
	}
}


/*******************************************************************************
Name......... StepReelBackwardsScreen1
Author....... Jon carroll
Date......... 27/07/05
Description.. Steps the reels down
Parameters... 1) The reel
Return....... None
History...... None
*******************************************************************************/

void StepReelBackwardsScreen1(UBYTE Reel)
{
	S1ReelPosition[Reel] = (S1ReelPosition[Reel] + PIXELS_PER_STEP1)
							% REEL_BAND_HEIGHTS1;
}


/*******************************************************************************
Name......... UpdateDisplay
Author....... Jon carroll
Date......... 27/07/05
Description.. update the animation
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void UpdateDisplayScreen1(void)
{
	UBYTE i;
										/* determine which state */
	switch (S1State)
	{
		case WAITING_STATE:
										/* determine if the timer has expired */
			if (S1StateTimer == 0)
			{
										/* set steps to go */
				S1StepsToGo[0] = 5;
				S1StepsToGo[1] = 5;
				S1StepsToGo[2] = 5;
				S1StepsToGo[3] = 5;
				S1StepsToGo[4] = 5;
										/* set the screen 1 state */
				S1State = STEP_BACK_STATE;
										/* kill the timer */
				SetTimer(&S1StateTimer, 0);
			}
		break;


		case STEP_BACK_STATE:
										/* determine if steps has
										   expired */
			if (S1StepsToGo[0] == 0)
			{
										/* set up new spin */
				S1StepsToGo[0] = 5;
				S1StepsToGo[1] = 5;
				S1StepsToGo[2] = 5;
				S1StepsToGo[3] = 5;
				S1StepsToGo[4] = 5;
										/* set the screen 1 state */
				S1State = STEP_FORE_STATE;
			}
			else
			{
										/* for all reels */
				for (i = 0; i < TOTAL_REELS_SCREEN1; i++)
				{
										/* step reels */
					StepReelBackwardsScreen1(i);
										/* decrement steps */
					S1StepsToGo[i]--;
				}
			}

		break;


		case STEP_FORE_STATE:
										/* determine if steps has
										   expired */
			if (S1StepsToGo[0] == 0)
			{
										/* set up new spin */
				S1StepsToGo[0] = 4 * (12 + 2);
				S1StepsToGo[1] = 4 * (12 + 4);
				S1StepsToGo[2] = 4 * (12 + 6);
				S1StepsToGo[3] = 4 * (12 + 8);
				S1StepsToGo[4] = 4 * (12 + 10);
										/* set the bonuce state */
				S1ReelBounceState[0] = NORMAL_STATE;
				S1ReelBounceState[1] = NORMAL_STATE;
				S1ReelBounceState[2] = NORMAL_STATE;
				S1ReelBounceState[3] = NORMAL_STATE;
				S1ReelBounceState[4] = NORMAL_STATE;
										/* set the screen 1 state */
				S1State = SPIN_STATE;
			}
			else
			{
										/* for all reels */
				for (i = 0; i < TOTAL_REELS_SCREEN1; i++)
				{
										/* step reels */
					StepReelForewardsScreen1(i);
										/* decrement steps */
					S1StepsToGo[i]--;
				}
			}

		break;

		case SPIN_STATE:
										/* determine if last reel
										   has stopped */
			if (S1StepsToGo[4] == 0 && S1ReelBounceState[4] == UP_STATE)
			{
										/* update state */
				S1State = WAITING_STATE;

										/* start the timer */
				SetTimer(&S1StateTimer, 300);
			}
			else
			{
										/* for all reels */
				for (i = 0; i < TOTAL_REELS_SCREEN1; i++)
				{
										/* determine which bounce state */
					switch (S1ReelBounceState[i])
					{
						case NORMAL_STATE:
										/* determine any steps left */
							if (S1StepsToGo[i])
							{
										/* advance reel */
								StepReelForewardsScreen1(i);
										/* decrement steps */
								S1StepsToGo[i]--;
							}
							else
							{
										/* update the state */
								S1StepsToGo[i] = 5;

								S1ReelBounceState[i] = DOWN_STATE;
							}

						break;

						case DOWN_STATE:

										/* determine any steps left */
							if (S1StepsToGo[i])
							{
										/* advance reel */
								StepReelForewardsScreen1(i);
										/* decrement steps */
								S1StepsToGo[i]--;
							}
							else
							{
										/* update the state */
								S1StepsToGo[i] = 5;

								S1ReelBounceState[i] = UP_STATE;
							}
						break;

										/* must be the up state */
						default:
										/* determine any steps left */
							if (S1StepsToGo[i])
							{
										/* advance reel */
								StepReelBackwardsScreen1(i);
										/* decrement steps */
								S1StepsToGo[i]--;
							}
						break;



					}
				}
			}

		break;

		default:
										/* update state */
				S1State = WAITING_STATE;

										/* start the timer */
				SetTimer(&S1StateTimer, 300);
		break;
	}
}


/*******************************************************************************
Name......... StepReelForeWardsScreen2
Author....... Jon carroll
Date......... 27/07/05
Description.. Steps the reels up
Parameters... 1) The reel
Return....... None
History...... None
*******************************************************************************/

void StepReelForewardsScreen2(UBYTE Reel)
{
	if (S2ReelPosition[Reel] < PIXELS_PER_STEP2)
	{
		S2ReelPosition[Reel] = (REEL_BAND_HEIGHTS2 - S2ReelPosition[Reel])
									- PIXELS_PER_STEP2;
	}
	else
	{
		S2ReelPosition[Reel] -= PIXELS_PER_STEP2;
	}
}


/*******************************************************************************
Name......... StepReelBackwardsScreen2
Author....... Jon carroll
Date......... 27/07/05
Description.. Steps the reels down
Parameters... 1) The reel
Return....... None
History...... None
*******************************************************************************/

void StepReelBackwardsScreen2(UBYTE Reel)
{
	S2ReelPosition[Reel] = (S2ReelPosition[Reel] + PIXELS_PER_STEP2)
							% REEL_BAND_HEIGHTS2;
}


/*******************************************************************************
Name......... UpdateDisplay
Author....... Jon carroll
Date......... 27/07/05
Description.. update the animation
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void UpdateDisplayScreen2(void)
{
	UBYTE i;
										/* determine which state */
	switch (S2State)
	{
		case WAITING_STATE:
										/* determine if the timer has expired */
			if (S2StateTimer == 0)
			{
										/* set steps to go */
				S2StepsToGo[0] = 5;
				S2StepsToGo[1] = 5;
				S2StepsToGo[2] = 5;
										/* set the screen 1 state */
				S2State = STEP_BACK_STATE;
										/* kill the timer */
				SetTimer(&S2StateTimer, 0);
			}
		break;


		case STEP_BACK_STATE:
										/* determine if steps has
										   expired */
			if (S2StepsToGo[0] == 0)
			{
										/* set up new spin */
				S2StepsToGo[0] = 5;
				S2StepsToGo[1] = 5;
				S2StepsToGo[2] = 5;
										/* set the screen 1 state */
				S2State = STEP_FORE_STATE;
			}
			else
			{
										/* for all reels */
				for (i = 0; i < TOTAL_REELS_SCREEN2; i++)
				{
										/* step reels */
					StepReelBackwardsScreen2(i);
										/* decrement steps */
					S2StepsToGo[i]--;
				}
			}

		break;


		case STEP_FORE_STATE:
										/* determine if steps has
										   expired */
			if (S2StepsToGo[0] == 0)
			{
										/* set up new spin */
				S2StepsToGo[0] = 8 * (12 + 2);
				S2StepsToGo[1] = 8 * (12 + 4);
				S2StepsToGo[2] = 8 * (12 + 6);
										/* set the bonuce state */
				S2ReelBounceState[0] = NORMAL_STATE;
				S2ReelBounceState[1] = NORMAL_STATE;
				S2ReelBounceState[2] = NORMAL_STATE;
										/* set the screen 1 state */
				S2State = SPIN_STATE;
			}
			else
			{
										/* for all reels */
				for (i = 0; i < TOTAL_REELS_SCREEN2; i++)
				{
										/* step reels */
					StepReelForewardsScreen2(i);
										/* decrement steps */
					S2StepsToGo[i]--;
				}
			}

		break;

		case SPIN_STATE:
										/* determine if last reel
										   has stopped */
			if (S2StepsToGo[(TOTAL_REELS_SCREEN2 - 1)] == 0 &&
				S2ReelBounceState[(TOTAL_REELS_SCREEN2 - 1)] == UP_STATE)
			{
										/* update state */
				S2State = WAITING_STATE;

										/* start the timer */
				SetTimer(&S2StateTimer, 300);
			}
			else
			{
										/* for all reels */
				for (i = 0; i < TOTAL_REELS_SCREEN2; i++)
				{
										/* determine which bounce state */
					switch (S2ReelBounceState[i])
					{
						case NORMAL_STATE:
										/* determine any steps left */
							if (S2StepsToGo[i])
							{
										/* advance reel */
								StepReelForewardsScreen2(i);
										/* decrement steps */
								S2StepsToGo[i]--;
							}
							else
							{
										/* update the state */
								S2StepsToGo[i] = 5;

								S2ReelBounceState[i] = DOWN_STATE;
							}

						break;

						case DOWN_STATE:

										/* determine any steps left */
							if (S2StepsToGo[i])
							{
										/* advance reel */
								StepReelForewardsScreen2(i);
										/* decrement steps */
								S2StepsToGo[i]--;
							}
							else
							{
										/* update the state */
								S2StepsToGo[i] = 5;

								S2ReelBounceState[i] = UP_STATE;
							}
						break;

										/* must be the up state */
						default:
										/* determine any steps left */
							if (S2StepsToGo[i])
							{
										/* advance reel */
								StepReelBackwardsScreen2(i);
										/* decrement steps */
								S2StepsToGo[i]--;
							}
						break;



					}
				}
			}

		break;

		default:
										/* update state */
				S2State = WAITING_STATE;

										/* start the timer */
				SetTimer(&S2StateTimer, 300);
		break;
	}
}

/*******************************************************************************
Name......... AdvanceFrames
Author....... Jon carroll
Date......... 27/07/05
Description.. Advnace both screens 1 frame
Parameters... None
Return....... None
History...... None
*******************************************************************************/

void AdvanceFrames(void)
{
												/* update the screen 1 */
	CalypsoFrameAdvance(screen1);
												/* wait for frame advance */
	WaitNFrames(screen1, 1);
												/* update the screen 2 */
	CalypsoFrameAdvance(screen2);
												/* wait for frame advance */
	WaitNFrames(screen2, 1);
}

/******************************************************************************/